@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,20 +13,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
 
+#ifndef __G_TLS_DATABASE_H__
+#define __G_TLS_DATABASE_H__
+
 #if !defined (__GIO_GIO_H_INSIDE__) && !defined (GIO_COMPILATION)
 #error "Only <gio/gio.h> can be included directly."
 #endif
 
-#ifndef __G_TLS_DATABASE_H__
-#define __G_TLS_DATABASE_H__
-
 #include <gio/giotypes.h>
 
 G_BEGIN_DECLS
@@ -145,8 +143,10 @@ struct _GTlsDatabaseClass
   gpointer padding[16];
 };
 
+GLIB_AVAILABLE_IN_ALL
 GType                g_tls_database_get_type                              (void) G_GNUC_CONST;
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificateFlags g_tls_database_verify_chain                          (GTlsDatabase            *self,
                                                                            GTlsCertificate         *chain,
                                                                            const gchar             *purpose,
@@ -156,6 +156,7 @@ GTlsCertificateFlags g_tls_database_verify_chain                          (GTlsD
                                                                            GCancellable            *cancellable,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 void                 g_tls_database_verify_chain_async                    (GTlsDatabase            *self,
                                                                            GTlsCertificate         *chain,
                                                                            const gchar             *purpose,
@@ -166,13 +167,16 @@ void                 g_tls_database_verify_chain_async                    (GTlsD
                                                                            GAsyncReadyCallback      callback,
                                                                            gpointer                 user_data);
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificateFlags g_tls_database_verify_chain_finish                   (GTlsDatabase            *self,
                                                                            GAsyncResult            *result,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 gchar*               g_tls_database_create_certificate_handle             (GTlsDatabase            *self,
                                                                            GTlsCertificate         *certificate);
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificate*     g_tls_database_lookup_certificate_for_handle         (GTlsDatabase            *self,
                                                                            const gchar             *handle,
                                                                            GTlsInteraction         *interaction,
@@ -180,6 +184,7 @@ GTlsCertificate*     g_tls_database_lookup_certificate_for_handle         (GTlsD
                                                                            GCancellable            *cancellable,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 void                 g_tls_database_lookup_certificate_for_handle_async   (GTlsDatabase            *self,
                                                                            const gchar             *handle,
                                                                            GTlsInteraction         *interaction,
@@ -188,10 +193,12 @@ void                 g_tls_database_lookup_certificate_for_handle_async   (GTlsD
                                                                            GAsyncReadyCallback      callback,
                                                                            gpointer                 user_data);
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificate*     g_tls_database_lookup_certificate_for_handle_finish  (GTlsDatabase            *self,
                                                                            GAsyncResult            *result,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificate*     g_tls_database_lookup_certificate_issuer             (GTlsDatabase            *self,
                                                                            GTlsCertificate         *certificate,
                                                                            GTlsInteraction         *interaction,
@@ -199,6 +206,7 @@ GTlsCertificate*     g_tls_database_lookup_certificate_issuer             (GTlsD
                                                                            GCancellable            *cancellable,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 void                 g_tls_database_lookup_certificate_issuer_async       (GTlsDatabase            *self,
                                                                            GTlsCertificate         *certificate,
                                                                            GTlsInteraction         *interaction,
@@ -207,10 +215,12 @@ void                 g_tls_database_lookup_certificate_issuer_async       (GTlsD
                                                                            GAsyncReadyCallback      callback,
                                                                            gpointer                 user_data);
 
+GLIB_AVAILABLE_IN_ALL
 GTlsCertificate*     g_tls_database_lookup_certificate_issuer_finish      (GTlsDatabase            *self,
                                                                            GAsyncResult            *result,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 GList*               g_tls_database_lookup_certificates_issued_by         (GTlsDatabase            *self,
                                                                            GByteArray              *issuer_raw_dn,
                                                                            GTlsInteraction         *interaction,
@@ -218,6 +228,7 @@ GList*               g_tls_database_lookup_certificates_issued_by         (GTlsD
                                                                            GCancellable            *cancellable,
                                                                            GError                 **error);
 
+GLIB_AVAILABLE_IN_ALL
 void                 g_tls_database_lookup_certificates_issued_by_async    (GTlsDatabase            *self,
                                                                             GByteArray              *issuer_raw_dn,
                                                                             GTlsInteraction         *interaction,
@@ -226,6 +237,7 @@ void                 g_tls_database_lookup_certificates_issued_by_async    (GTls
                                                                             GAsyncReadyCallback      callback,
                                                                             gpointer                 user_data);
 
+GLIB_AVAILABLE_IN_ALL
 GList*               g_tls_database_lookup_certificates_issued_by_finish   (GTlsDatabase            *self,
                                                                             GAsyncResult            *result,
                                                                             GError                 **error);
