@@ -1,10 +1,10 @@
 /* 
  * Copyright © 2007 Ryan Lortie
  * 
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  * 
  * See the included COPYING file for more information.
  */
@@ -96,6 +96,9 @@ static struct test tests[] =
   { "<bool mb='t' ob='f' tri='1'/>", "<bool(1) 1 0 1>" },
   { "<bool mb='y' ob='n' tri='0'/>", "<bool(1) 1 0 0>" },
 
+  { "<bool mb='y' my:attr='q'><my:tag/></bool>", "<bool(1) 1 0 -1>" },
+  { "<bool mb='y' my:attr='q'><my:tag>some <b>text</b> is in here</my:tag></bool>", "<bool(1) 1 0 -1>" },
+
   { "<bool ob='y'/>", "<bool(0) 0 0 -1>",
     G_MARKUP_ERROR_MISSING_ATTRIBUTE, "'mb'" },
 
@@ -147,7 +150,7 @@ test_collect (gconstpointer d)
   gboolean result;
 
   string = g_string_new ("");
-  ctx = g_markup_parse_context_new (&parser, 0, string, NULL);
+  ctx = g_markup_parse_context_new (&parser, G_MARKUP_IGNORE_QUALIFIED, string, NULL);
   result = g_markup_parse_context_parse (ctx,
                                          test->document,
                                          -1, &error);
@@ -197,22 +200,20 @@ static GMarkupParser cleanup_parser = {
 static void
 test_cleanup (void)
 {
+  GMarkupParseContext *context;
+
   if (!g_test_undefined ())
     return;
 
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
-    {
-      GMarkupParseContext *context;
+  context = g_markup_parse_context_new (&cleanup_parser, 0, NULL, NULL);
+  g_markup_parse_context_parse (context, XML, -1, NULL);
 
-      context = g_markup_parse_context_new (&cleanup_parser, 0, NULL, NULL);
-      g_markup_parse_context_parse (context, XML, -1, NULL);
-      g_markup_parse_context_end_parse (context, NULL);
-      g_markup_parse_context_free (context);
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL,
+                         "g_markup_parse_context_end_parse: assertion 'context->state != STATE_ERROR' failed");
+  g_markup_parse_context_end_parse (context, NULL);
+  g_test_assert_expected_messages ();
 
-      exit (0);
-    }
-  g_test_trap_assert_failed ();
-  g_test_trap_assert_stderr ("*assertion `context->state != STATE_ERROR' failed*");
+  g_markup_parse_context_free (context);
 }
 
 int
